r"""
    This code was generated by
   ___ _ _ _ _ _    _ ____    ____ ____ _    ____ ____ _  _ ____ ____ ____ ___ __   __
    |  | | | | |    | |  | __ |  | |__| | __ | __ |___ |\ | |___ |__/ |__|  | |  | |__/
    |  |_|_| | |___ | |__|    |__| |  | |    |__] |___ | \| |___ |  \ |  |  | |__| |  \

    Twilio - Numbers
    This is the public Twilio REST API.

    NOTE: This class is auto generated by OpenAPI Generator.
    https://openapi-generator.tech
    Do not edit the class manually.
"""

from typing import Any, Dict, Optional, Union
from twilio.base import deserialize, values
from twilio.base.instance_context import InstanceContext
from twilio.base.instance_resource import InstanceResource
from twilio.base.list_resource import ListResource
from twilio.base.version import Version


class PortingPortabilityInstance(InstanceResource):

    class NumberType(object):
        LOCAL = "LOCAL"
        UNKNOWN = "UNKNOWN"
        MOBILE = "MOBILE"
        TOLL_FREE = "TOLL-FREE"

    """
    :ivar phone_number: The phone number which portability is to be checked. Phone numbers are in E.164 format (e.g. +16175551212).
    :ivar account_sid: The target account sid to which the number will be ported
    :ivar portable: Boolean flag specifying if phone number is portable or not.
    :ivar pin_and_account_number_required: Boolean flag specifying if PIN and account number is required for the phone number.
    :ivar not_portable_reason: Reason why the phone number cannot be ported into Twilio, `null` otherwise.
    :ivar not_portable_reason_code: The Portability Reason Code for the phone number if it cannot be ported into Twilio, `null` otherwise. One of `22131`, `22132`, `22130`, `22133`, `22102` or `22135`.
    :ivar number_type: 
    :ivar country: Country the phone number belongs to.
    :ivar messaging_carrier: Current messaging carrier of the phone number
    :ivar voice_carrier: Current voice carrier of the phone number
    :ivar url: This is the url of the request that you're trying to reach out to locate the resource.
    """

    def __init__(
        self,
        version: Version,
        payload: Dict[str, Any],
        phone_number: Optional[str] = None,
    ):
        super().__init__(version)

        self.phone_number: Optional[str] = payload.get("phone_number")
        self.account_sid: Optional[str] = payload.get("account_sid")
        self.portable: Optional[bool] = payload.get("portable")
        self.pin_and_account_number_required: Optional[bool] = payload.get(
            "pin_and_account_number_required"
        )
        self.not_portable_reason: Optional[str] = payload.get("not_portable_reason")
        self.not_portable_reason_code: Optional[int] = deserialize.integer(
            payload.get("not_portable_reason_code")
        )
        self.number_type: Optional["PortingPortabilityInstance.NumberType"] = (
            payload.get("number_type")
        )
        self.country: Optional[str] = payload.get("country")
        self.messaging_carrier: Optional[str] = payload.get("messaging_carrier")
        self.voice_carrier: Optional[str] = payload.get("voice_carrier")
        self.url: Optional[str] = payload.get("url")

        self._solution = {
            "phone_number": phone_number or self.phone_number,
        }
        self._context: Optional[PortingPortabilityContext] = None

    @property
    def _proxy(self) -> "PortingPortabilityContext":
        """
        Generate an instance context for the instance, the context is capable of
        performing various actions. All instance actions are proxied to the context

        :returns: PortingPortabilityContext for this PortingPortabilityInstance
        """
        if self._context is None:
            self._context = PortingPortabilityContext(
                self._version,
                phone_number=self._solution["phone_number"],
            )
        return self._context

    def fetch(
        self, target_account_sid: Union[str, object] = values.unset
    ) -> "PortingPortabilityInstance":
        """
        Fetch the PortingPortabilityInstance

        :param target_account_sid: The SID of the account where the phone number(s) will be ported.

        :returns: The fetched PortingPortabilityInstance
        """
        return self._proxy.fetch(
            target_account_sid=target_account_sid,
        )

    async def fetch_async(
        self, target_account_sid: Union[str, object] = values.unset
    ) -> "PortingPortabilityInstance":
        """
        Asynchronous coroutine to fetch the PortingPortabilityInstance

        :param target_account_sid: The SID of the account where the phone number(s) will be ported.

        :returns: The fetched PortingPortabilityInstance
        """
        return await self._proxy.fetch_async(
            target_account_sid=target_account_sid,
        )

    def __repr__(self) -> str:
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        """
        context = " ".join("{}={}".format(k, v) for k, v in self._solution.items())
        return "<Twilio.Numbers.V1.PortingPortabilityInstance {}>".format(context)


class PortingPortabilityContext(InstanceContext):

    def __init__(self, version: Version, phone_number: str):
        """
        Initialize the PortingPortabilityContext

        :param version: Version that contains the resource
        :param phone_number: The phone number which portability is to be checked. Phone numbers are in E.164 format (e.g. +16175551212).
        """
        super().__init__(version)

        # Path Solution
        self._solution = {
            "phone_number": phone_number,
        }
        self._uri = "/Porting/Portability/PhoneNumber/{phone_number}".format(
            **self._solution
        )

    def fetch(
        self, target_account_sid: Union[str, object] = values.unset
    ) -> PortingPortabilityInstance:
        """
        Fetch the PortingPortabilityInstance

        :param target_account_sid: The SID of the account where the phone number(s) will be ported.

        :returns: The fetched PortingPortabilityInstance
        """

        data = values.of(
            {
                "TargetAccountSid": target_account_sid,
            }
        )

        payload = self._version.fetch(method="GET", uri=self._uri, params=data)

        return PortingPortabilityInstance(
            self._version,
            payload,
            phone_number=self._solution["phone_number"],
        )

    async def fetch_async(
        self, target_account_sid: Union[str, object] = values.unset
    ) -> PortingPortabilityInstance:
        """
        Asynchronous coroutine to fetch the PortingPortabilityInstance

        :param target_account_sid: The SID of the account where the phone number(s) will be ported.

        :returns: The fetched PortingPortabilityInstance
        """

        data = values.of(
            {
                "TargetAccountSid": target_account_sid,
            }
        )

        payload = await self._version.fetch_async(
            method="GET", uri=self._uri, params=data
        )

        return PortingPortabilityInstance(
            self._version,
            payload,
            phone_number=self._solution["phone_number"],
        )

    def __repr__(self) -> str:
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        """
        context = " ".join("{}={}".format(k, v) for k, v in self._solution.items())
        return "<Twilio.Numbers.V1.PortingPortabilityContext {}>".format(context)


class PortingPortabilityList(ListResource):

    def __init__(self, version: Version):
        """
        Initialize the PortingPortabilityList

        :param version: Version that contains the resource

        """
        super().__init__(version)

    def get(self, phone_number: str) -> PortingPortabilityContext:
        """
        Constructs a PortingPortabilityContext

        :param phone_number: The phone number which portability is to be checked. Phone numbers are in E.164 format (e.g. +16175551212).
        """
        return PortingPortabilityContext(self._version, phone_number=phone_number)

    def __call__(self, phone_number: str) -> PortingPortabilityContext:
        """
        Constructs a PortingPortabilityContext

        :param phone_number: The phone number which portability is to be checked. Phone numbers are in E.164 format (e.g. +16175551212).
        """
        return PortingPortabilityContext(self._version, phone_number=phone_number)

    def __repr__(self) -> str:
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        """
        return "<Twilio.Numbers.V1.PortingPortabilityList>"



r"""
    This code was generated by
   ___ _ _ _ _ _    _ ____    ____ ____ _    ____ ____ _  _ ____ ____ ____ ___ __   __
    |  | | | | |    | |  | __ |  | |__| | __ | __ |___ |\ | |___ |__/ |__|  | |  | |__/
    |  |_|_| | |___ | |__|    |__| |  | |    |__] |___ | \| |___ |  \ |  |  | |__| |  \

    Twilio - Api
    This is the public Twilio REST API.

    NOTE: This class is auto generated by OpenAPI Generator.
    https://openapi-generator.tech
    Do not edit the class manually.
"""

from datetime import datetime
from typing import Any, Dict, List, Optional, Union, Iterator, AsyncIterator
from twilio.base import deserialize, values
from twilio.base.instance_context import InstanceContext
from twilio.base.instance_resource import InstanceResource
from twilio.base.list_resource import ListResource
from twilio.base.version import Version
from twilio.base.page import Page


class IpAddressInstance(InstanceResource):
    """
    :ivar sid: A 34 character string that uniquely identifies this resource.
    :ivar account_sid: The unique id of the Account that is responsible for this resource.
    :ivar friendly_name: A human readable descriptive text for this resource, up to 255 characters long.
    :ivar ip_address: An IP address in dotted decimal notation from which you want to accept traffic. Any SIP requests from this IP address will be allowed by Twilio. IPv4 only supported today.
    :ivar cidr_prefix_length: An integer representing the length of the CIDR prefix to use with this IP address when accepting traffic. By default the entire IP address is used.
    :ivar ip_access_control_list_sid: The unique id of the IpAccessControlList resource that includes this resource.
    :ivar date_created: The date that this resource was created, given as GMT in [RFC 2822](https://www.php.net/manual/en/class.datetime.php#datetime.constants.rfc2822) format.
    :ivar date_updated: The date that this resource was last updated, given as GMT in [RFC 2822](https://www.php.net/manual/en/class.datetime.php#datetime.constants.rfc2822) format.
    :ivar uri: The URI for this resource, relative to `https://api.twilio.com`
    """

    def __init__(
        self,
        version: Version,
        payload: Dict[str, Any],
        account_sid: str,
        ip_access_control_list_sid: str,
        sid: Optional[str] = None,
    ):
        super().__init__(version)

        self.sid: Optional[str] = payload.get("sid")
        self.account_sid: Optional[str] = payload.get("account_sid")
        self.friendly_name: Optional[str] = payload.get("friendly_name")
        self.ip_address: Optional[str] = payload.get("ip_address")
        self.cidr_prefix_length: Optional[int] = deserialize.integer(
            payload.get("cidr_prefix_length")
        )
        self.ip_access_control_list_sid: Optional[str] = payload.get(
            "ip_access_control_list_sid"
        )
        self.date_created: Optional[datetime] = deserialize.rfc2822_datetime(
            payload.get("date_created")
        )
        self.date_updated: Optional[datetime] = deserialize.rfc2822_datetime(
            payload.get("date_updated")
        )
        self.uri: Optional[str] = payload.get("uri")

        self._solution = {
            "account_sid": account_sid,
            "ip_access_control_list_sid": ip_access_control_list_sid,
            "sid": sid or self.sid,
        }
        self._context: Optional[IpAddressContext] = None

    @property
    def _proxy(self) -> "IpAddressContext":
        """
        Generate an instance context for the instance, the context is capable of
        performing various actions. All instance actions are proxied to the context

        :returns: IpAddressContext for this IpAddressInstance
        """
        if self._context is None:
            self._context = IpAddressContext(
                self._version,
                account_sid=self._solution["account_sid"],
                ip_access_control_list_sid=self._solution["ip_access_control_list_sid"],
                sid=self._solution["sid"],
            )
        return self._context

    def delete(self) -> bool:
        """
        Deletes the IpAddressInstance


        :returns: True if delete succeeds, False otherwise
        """
        return self._proxy.delete()

    async def delete_async(self) -> bool:
        """
        Asynchronous coroutine that deletes the IpAddressInstance


        :returns: True if delete succeeds, False otherwise
        """
        return await self._proxy.delete_async()

    def fetch(self) -> "IpAddressInstance":
        """
        Fetch the IpAddressInstance


        :returns: The fetched IpAddressInstance
        """
        return self._proxy.fetch()

    async def fetch_async(self) -> "IpAddressInstance":
        """
        Asynchronous coroutine to fetch the IpAddressInstance


        :returns: The fetched IpAddressInstance
        """
        return await self._proxy.fetch_async()

    def update(
        self,
        ip_address: Union[str, object] = values.unset,
        friendly_name: Union[str, object] = values.unset,
        cidr_prefix_length: Union[int, object] = values.unset,
    ) -> "IpAddressInstance":
        """
        Update the IpAddressInstance

        :param ip_address: An IP address in dotted decimal notation from which you want to accept traffic. Any SIP requests from this IP address will be allowed by Twilio. IPv4 only supported today.
        :param friendly_name: A human readable descriptive text for this resource, up to 255 characters long.
        :param cidr_prefix_length: An integer representing the length of the CIDR prefix to use with this IP address when accepting traffic. By default the entire IP address is used.

        :returns: The updated IpAddressInstance
        """
        return self._proxy.update(
            ip_address=ip_address,
            friendly_name=friendly_name,
            cidr_prefix_length=cidr_prefix_length,
        )

    async def update_async(
        self,
        ip_address: Union[str, object] = values.unset,
        friendly_name: Union[str, object] = values.unset,
        cidr_prefix_length: Union[int, object] = values.unset,
    ) -> "IpAddressInstance":
        """
        Asynchronous coroutine to update the IpAddressInstance

        :param ip_address: An IP address in dotted decimal notation from which you want to accept traffic. Any SIP requests from this IP address will be allowed by Twilio. IPv4 only supported today.
        :param friendly_name: A human readable descriptive text for this resource, up to 255 characters long.
        :param cidr_prefix_length: An integer representing the length of the CIDR prefix to use with this IP address when accepting traffic. By default the entire IP address is used.

        :returns: The updated IpAddressInstance
        """
        return await self._proxy.update_async(
            ip_address=ip_address,
            friendly_name=friendly_name,
            cidr_prefix_length=cidr_prefix_length,
        )

    def __repr__(self) -> str:
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        """
        context = " ".join("{}={}".format(k, v) for k, v in self._solution.items())
        return "<Twilio.Api.V2010.IpAddressInstance {}>".format(context)


class IpAddressContext(InstanceContext):

    def __init__(
        self,
        version: Version,
        account_sid: str,
        ip_access_control_list_sid: str,
        sid: str,
    ):
        """
        Initialize the IpAddressContext

        :param version: Version that contains the resource
        :param account_sid: The unique id of the [Account](https://www.twilio.com/docs/iam/api/account) responsible for this resource.
        :param ip_access_control_list_sid: The IpAccessControlList Sid that identifies the IpAddress resources to update.
        :param sid: A 34 character string that identifies the IpAddress resource to update.
        """
        super().__init__(version)

        # Path Solution
        self._solution = {
            "account_sid": account_sid,
            "ip_access_control_list_sid": ip_access_control_list_sid,
            "sid": sid,
        }
        self._uri = "/Accounts/{account_sid}/SIP/IpAccessControlLists/{ip_access_control_list_sid}/IpAddresses/{sid}.json".format(
            **self._solution
        )

    def delete(self) -> bool:
        """
        Deletes the IpAddressInstance


        :returns: True if delete succeeds, False otherwise
        """
        return self._version.delete(
            method="DELETE",
            uri=self._uri,
        )

    async def delete_async(self) -> bool:
        """
        Asynchronous coroutine that deletes the IpAddressInstance


        :returns: True if delete succeeds, False otherwise
        """
        return await self._version.delete_async(
            method="DELETE",
            uri=self._uri,
        )

    def fetch(self) -> IpAddressInstance:
        """
        Fetch the IpAddressInstance


        :returns: The fetched IpAddressInstance
        """

        payload = self._version.fetch(
            method="GET",
            uri=self._uri,
        )

        return IpAddressInstance(
            self._version,
            payload,
            account_sid=self._solution["account_sid"],
            ip_access_control_list_sid=self._solution["ip_access_control_list_sid"],
            sid=self._solution["sid"],
        )

    async def fetch_async(self) -> IpAddressInstance:
        """
        Asynchronous coroutine to fetch the IpAddressInstance


        :returns: The fetched IpAddressInstance
        """

        payload = await self._version.fetch_async(
            method="GET",
            uri=self._uri,
        )

        return IpAddressInstance(
            self._version,
            payload,
            account_sid=self._solution["account_sid"],
            ip_access_control_list_sid=self._solution["ip_access_control_list_sid"],
            sid=self._solution["sid"],
        )

    def update(
        self,
        ip_address: Union[str, object] = values.unset,
        friendly_name: Union[str, object] = values.unset,
        cidr_prefix_length: Union[int, object] = values.unset,
    ) -> IpAddressInstance:
        """
        Update the IpAddressInstance

        :param ip_address: An IP address in dotted decimal notation from which you want to accept traffic. Any SIP requests from this IP address will be allowed by Twilio. IPv4 only supported today.
        :param friendly_name: A human readable descriptive text for this resource, up to 255 characters long.
        :param cidr_prefix_length: An integer representing the length of the CIDR prefix to use with this IP address when accepting traffic. By default the entire IP address is used.

        :returns: The updated IpAddressInstance
        """
        data = values.of(
            {
                "IpAddress": ip_address,
                "FriendlyName": friendly_name,
                "CidrPrefixLength": cidr_prefix_length,
            }
        )

        payload = self._version.update(
            method="POST",
            uri=self._uri,
            data=data,
        )

        return IpAddressInstance(
            self._version,
            payload,
            account_sid=self._solution["account_sid"],
            ip_access_control_list_sid=self._solution["ip_access_control_list_sid"],
            sid=self._solution["sid"],
        )

    async def update_async(
        self,
        ip_address: Union[str, object] = values.unset,
        friendly_name: Union[str, object] = values.unset,
        cidr_prefix_length: Union[int, object] = values.unset,
    ) -> IpAddressInstance:
        """
        Asynchronous coroutine to update the IpAddressInstance

        :param ip_address: An IP address in dotted decimal notation from which you want to accept traffic. Any SIP requests from this IP address will be allowed by Twilio. IPv4 only supported today.
        :param friendly_name: A human readable descriptive text for this resource, up to 255 characters long.
        :param cidr_prefix_length: An integer representing the length of the CIDR prefix to use with this IP address when accepting traffic. By default the entire IP address is used.

        :returns: The updated IpAddressInstance
        """
        data = values.of(
            {
                "IpAddress": ip_address,
                "FriendlyName": friendly_name,
                "CidrPrefixLength": cidr_prefix_length,
            }
        )

        payload = await self._version.update_async(
            method="POST",
            uri=self._uri,
            data=data,
        )

        return IpAddressInstance(
            self._version,
            payload,
            account_sid=self._solution["account_sid"],
            ip_access_control_list_sid=self._solution["ip_access_control_list_sid"],
            sid=self._solution["sid"],
        )

    def __repr__(self) -> str:
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        """
        context = " ".join("{}={}".format(k, v) for k, v in self._solution.items())
        return "<Twilio.Api.V2010.IpAddressContext {}>".format(context)


class IpAddressPage(Page):

    def get_instance(self, payload: Dict[str, Any]) -> IpAddressInstance:
        """
        Build an instance of IpAddressInstance

        :param payload: Payload response from the API
        """
        return IpAddressInstance(
            self._version,
            payload,
            account_sid=self._solution["account_sid"],
            ip_access_control_list_sid=self._solution["ip_access_control_list_sid"],
        )

    def __repr__(self) -> str:
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        """
        return "<Twilio.Api.V2010.IpAddressPage>"


class IpAddressList(ListResource):

    def __init__(
        self, version: Version, account_sid: str, ip_access_control_list_sid: str
    ):
        """
        Initialize the IpAddressList

        :param version: Version that contains the resource
        :param account_sid: The unique id of the [Account](https://www.twilio.com/docs/iam/api/account) responsible for this resource.
        :param ip_access_control_list_sid: The IpAccessControlList Sid that identifies the IpAddress resources to read.

        """
        super().__init__(version)

        # Path Solution
        self._solution = {
            "account_sid": account_sid,
            "ip_access_control_list_sid": ip_access_control_list_sid,
        }
        self._uri = "/Accounts/{account_sid}/SIP/IpAccessControlLists/{ip_access_control_list_sid}/IpAddresses.json".format(
            **self._solution
        )

    def create(
        self,
        friendly_name: str,
        ip_address: str,
        cidr_prefix_length: Union[int, object] = values.unset,
    ) -> IpAddressInstance:
        """
        Create the IpAddressInstance

        :param friendly_name: A human readable descriptive text for this resource, up to 255 characters long.
        :param ip_address: An IP address in dotted decimal notation from which you want to accept traffic. Any SIP requests from this IP address will be allowed by Twilio. IPv4 only supported today.
        :param cidr_prefix_length: An integer representing the length of the CIDR prefix to use with this IP address when accepting traffic. By default the entire IP address is used.

        :returns: The created IpAddressInstance
        """

        data = values.of(
            {
                "FriendlyName": friendly_name,
                "IpAddress": ip_address,
                "CidrPrefixLength": cidr_prefix_length,
            }
        )

        payload = self._version.create(
            method="POST",
            uri=self._uri,
            data=data,
        )

        return IpAddressInstance(
            self._version,
            payload,
            account_sid=self._solution["account_sid"],
            ip_access_control_list_sid=self._solution["ip_access_control_list_sid"],
        )

    async def create_async(
        self,
        friendly_name: str,
        ip_address: str,
        cidr_prefix_length: Union[int, object] = values.unset,
    ) -> IpAddressInstance:
        """
        Asynchronously create the IpAddressInstance

        :param friendly_name: A human readable descriptive text for this resource, up to 255 characters long.
        :param ip_address: An IP address in dotted decimal notation from which you want to accept traffic. Any SIP requests from this IP address will be allowed by Twilio. IPv4 only supported today.
        :param cidr_prefix_length: An integer representing the length of the CIDR prefix to use with this IP address when accepting traffic. By default the entire IP address is used.

        :returns: The created IpAddressInstance
        """

        data = values.of(
            {
                "FriendlyName": friendly_name,
                "IpAddress": ip_address,
                "CidrPrefixLength": cidr_prefix_length,
            }
        )

        payload = await self._version.create_async(
            method="POST",
            uri=self._uri,
            data=data,
        )

        return IpAddressInstance(
            self._version,
            payload,
            account_sid=self._solution["account_sid"],
            ip_access_control_list_sid=self._solution["ip_access_control_list_sid"],
        )

    def stream(
        self,
        limit: Optional[int] = None,
        page_size: Optional[int] = None,
    ) -> Iterator[IpAddressInstance]:
        """
        Streams IpAddressInstance records from the API as a generator stream.
        This operation lazily loads records as efficiently as possible until the limit
        is reached.
        The results are returned as a generator, so this operation is memory efficient.

        :param limit: Upper limit for the number of records to return. stream()
                      guarantees to never return more than limit.  Default is no limit
        :param page_size: Number of records to fetch per request, when not set will use
                          the default value of 50 records.  If no page_size is defined
                          but a limit is defined, stream() will attempt to read the
                          limit with the most efficient page size, i.e. min(limit, 1000)

        :returns: Generator that will yield up to limit results
        """
        limits = self._version.read_limits(limit, page_size)
        page = self.page(page_size=limits["page_size"])

        return self._version.stream(page, limits["limit"])

    async def stream_async(
        self,
        limit: Optional[int] = None,
        page_size: Optional[int] = None,
    ) -> AsyncIterator[IpAddressInstance]:
        """
        Asynchronously streams IpAddressInstance records from the API as a generator stream.
        This operation lazily loads records as efficiently as possible until the limit
        is reached.
        The results are returned as a generator, so this operation is memory efficient.

        :param limit: Upper limit for the number of records to return. stream()
                      guarantees to never return more than limit.  Default is no limit
        :param page_size: Number of records to fetch per request, when not set will use
                          the default value of 50 records.  If no page_size is defined
                          but a limit is defined, stream() will attempt to read the
                          limit with the most efficient page size, i.e. min(limit, 1000)

        :returns: Generator that will yield up to limit results
        """
        limits = self._version.read_limits(limit, page_size)
        page = await self.page_async(page_size=limits["page_size"])

        return self._version.stream_async(page, limits["limit"])

    def list(
        self,
        limit: Optional[int] = None,
        page_size: Optional[int] = None,
    ) -> List[IpAddressInstance]:
        """
        Lists IpAddressInstance records from the API as a list.
        Unlike stream(), this operation is eager and will load `limit` records into
        memory before returning.

        :param limit: Upper limit for the number of records to return. list() guarantees
                      never to return more than limit.  Default is no limit
        :param page_size: Number of records to fetch per request, when not set will use
                          the default value of 50 records.  If no page_size is defined
                          but a limit is defined, list() will attempt to read the limit
                          with the most efficient page size, i.e. min(limit, 1000)

        :returns: list that will contain up to limit results
        """
        return list(
            self.stream(
                limit=limit,
                page_size=page_size,
            )
        )

    async def list_async(
        self,
        limit: Optional[int] = None,
        page_size: Optional[int] = None,
    ) -> List[IpAddressInstance]:
        """
        Asynchronously lists IpAddressInstance records from the API as a list.
        Unlike stream(), this operation is eager and will load `limit` records into
        memory before returning.

        :param limit: Upper limit for the number of records to return. list() guarantees
                      never to return more than limit.  Default is no limit
        :param page_size: Number of records to fetch per request, when not set will use
                          the default value of 50 records.  If no page_size is defined
                          but a limit is defined, list() will attempt to read the limit
                          with the most efficient page size, i.e. min(limit, 1000)

        :returns: list that will contain up to limit results
        """
        return [
            record
            async for record in await self.stream_async(
                limit=limit,
                page_size=page_size,
            )
        ]

    def page(
        self,
        page_token: Union[str, object] = values.unset,
        page_number: Union[int, object] = values.unset,
        page_size: Union[int, object] = values.unset,
    ) -> IpAddressPage:
        """
        Retrieve a single page of IpAddressInstance records from the API.
        Request is executed immediately

        :param page_token: PageToken provided by the API
        :param page_number: Page Number, this value is simply for client state
        :param page_size: Number of records to return, defaults to 50

        :returns: Page of IpAddressInstance
        """
        data = values.of(
            {
                "PageToken": page_token,
                "Page": page_number,
                "PageSize": page_size,
            }
        )

        response = self._version.page(method="GET", uri=self._uri, params=data)
        return IpAddressPage(self._version, response, self._solution)

    async def page_async(
        self,
        page_token: Union[str, object] = values.unset,
        page_number: Union[int, object] = values.unset,
        page_size: Union[int, object] = values.unset,
    ) -> IpAddressPage:
        """
        Asynchronously retrieve a single page of IpAddressInstance records from the API.
        Request is executed immediately

        :param page_token: PageToken provided by the API
        :param page_number: Page Number, this value is simply for client state
        :param page_size: Number of records to return, defaults to 50

        :returns: Page of IpAddressInstance
        """
        data = values.of(
            {
                "PageToken": page_token,
                "Page": page_number,
                "PageSize": page_size,
            }
        )

        response = await self._version.page_async(
            method="GET", uri=self._uri, params=data
        )
        return IpAddressPage(self._version, response, self._solution)

    def get_page(self, target_url: str) -> IpAddressPage:
        """
        Retrieve a specific page of IpAddressInstance records from the API.
        Request is executed immediately

        :param target_url: API-generated URL for the requested results page

        :returns: Page of IpAddressInstance
        """
        response = self._version.domain.twilio.request("GET", target_url)
        return IpAddressPage(self._version, response, self._solution)

    async def get_page_async(self, target_url: str) -> IpAddressPage:
        """
        Asynchronously retrieve a specific page of IpAddressInstance records from the API.
        Request is executed immediately

        :param target_url: API-generated URL for the requested results page

        :returns: Page of IpAddressInstance
        """
        response = await self._version.domain.twilio.request_async("GET", target_url)
        return IpAddressPage(self._version, response, self._solution)

    def get(self, sid: str) -> IpAddressContext:
        """
        Constructs a IpAddressContext

        :param sid: A 34 character string that identifies the IpAddress resource to update.
        """
        return IpAddressContext(
            self._version,
            account_sid=self._solution["account_sid"],
            ip_access_control_list_sid=self._solution["ip_access_control_list_sid"],
            sid=sid,
        )

    def __call__(self, sid: str) -> IpAddressContext:
        """
        Constructs a IpAddressContext

        :param sid: A 34 character string that identifies the IpAddress resource to update.
        """
        return IpAddressContext(
            self._version,
            account_sid=self._solution["account_sid"],
            ip_access_control_list_sid=self._solution["ip_access_control_list_sid"],
            sid=sid,
        )

    def __repr__(self) -> str:
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        """
        return "<Twilio.Api.V2010.IpAddressList>"



r"""
    This code was generated by
   ___ _ _ _ _ _    _ ____    ____ ____ _    ____ ____ _  _ ____ ____ ____ ___ __   __
    |  | | | | |    | |  | __ |  | |__| | __ | __ |___ |\ | |___ |__/ |__|  | |  | |__/
    |  |_|_| | |___ | |__|    |__| |  | |    |__] |___ | \| |___ |  \ |  |  | |__| |  \

    Bulk Messaging and Broadcast
    Bulk Sending is a public Twilio REST API for 1:Many Message creation up to 100 recipients. Broadcast is a public Twilio REST API for 1:Many Message creation up to 10,000 recipients via file upload.

    NOTE: This class is auto generated by OpenAPI Generator.
    https://openapi-generator.tech
    Do not edit the class manually.
"""

from typing import Optional
from twilio.base.version import Version
from twilio.base.domain import Domain
from twilio.rest.preview_messaging.v1.broadcast import BroadcastList
from twilio.rest.preview_messaging.v1.message import MessageList


class V1(Version):

    def __init__(self, domain: Domain):
        """
        Initialize the V1 version of PreviewMessaging

        :param domain: The Twilio.preview_messaging domain
        """
        super().__init__(domain, "v1")
        self._broadcasts: Optional[BroadcastList] = None
        self._messages: Optional[MessageList] = None

    @property
    def broadcasts(self) -> BroadcastList:
        if self._broadcasts is None:
            self._broadcasts = BroadcastList(self)
        return self._broadcasts

    @property
    def messages(self) -> MessageList:
        if self._messages is None:
            self._messages = MessageList(self)
        return self._messages

    def __repr__(self) -> str:
        """
        Provide a friendly representation
        :returns: Machine friendly representation
        """
        return "<Twilio.PreviewMessaging.V1>"



r"""
    This code was generated by
   ___ _ _ _ _ _    _ ____    ____ ____ _    ____ ____ _  _ ____ ____ ____ ___ __   __
    |  | | | | |    | |  | __ |  | |__| | __ | __ |___ |\ | |___ |__/ |__|  | |  | |__/
    |  |_|_| | |___ | |__|    |__| |  | |    |__] |___ | \| |___ |  \ |  |  | |__| |  \

    Twilio - Numbers
    This is the public Twilio REST API.

    NOTE: This class is auto generated by OpenAPI Generator.
    https://openapi-generator.tech
    Do not edit the class manually.
"""

from datetime import datetime
from typing import Any, Dict, List, Optional
from twilio.base import deserialize, serialize, values
from twilio.base.instance_context import InstanceContext
from twilio.base.instance_resource import InstanceResource
from twilio.base.list_resource import ListResource
from twilio.base.version import Version


class PortingBulkPortabilityInstance(InstanceResource):

    class Status(object):
        IN_PROGRESS = "in-progress"
        COMPLETED = "completed"
        EXPIRED = "expired"

    """
    :ivar sid: A 34 character string that uniquely identifies this Portability check.
    :ivar status: 
    :ivar datetime_created: The date that the Portability check was created, given in ISO 8601 format.
    :ivar phone_numbers: Contains a list with all the information of the requested phone numbers. Each phone number contains the following properties: `phone_number`: The phone number which portability is to be checked. `portable`: Boolean flag specifying if phone number is portable or not. `not_portable_reason`: Reason why the phone number cannot be ported into Twilio, `null` otherwise. `not_portable_reason_code`: The Portability Reason Code for the phone number if it cannot be ported in Twilio, `null` otherwise. `pin_and_account_number_required`: Boolean flag specifying if PIN and account number is required for the phone number. `number_type`: The type of the requested phone number. `country` Country the phone number belongs to. `messaging_carrier` Current messaging carrier of the phone number. `voice_carrier` Current voice carrier of the phone number.
    :ivar url: This is the url of the request that you're trying to reach out to locate the resource.
    """

    def __init__(
        self, version: Version, payload: Dict[str, Any], sid: Optional[str] = None
    ):
        super().__init__(version)

        self.sid: Optional[str] = payload.get("sid")
        self.status: Optional["PortingBulkPortabilityInstance.Status"] = payload.get(
            "status"
        )
        self.datetime_created: Optional[datetime] = deserialize.iso8601_datetime(
            payload.get("datetime_created")
        )
        self.phone_numbers: Optional[List[Dict[str, object]]] = payload.get(
            "phone_numbers"
        )
        self.url: Optional[str] = payload.get("url")

        self._solution = {
            "sid": sid or self.sid,
        }
        self._context: Optional[PortingBulkPortabilityContext] = None

    @property
    def _proxy(self) -> "PortingBulkPortabilityContext":
        """
        Generate an instance context for the instance, the context is capable of
        performing various actions. All instance actions are proxied to the context

        :returns: PortingBulkPortabilityContext for this PortingBulkPortabilityInstance
        """
        if self._context is None:
            self._context = PortingBulkPortabilityContext(
                self._version,
                sid=self._solution["sid"],
            )
        return self._context

    def fetch(self) -> "PortingBulkPortabilityInstance":
        """
        Fetch the PortingBulkPortabilityInstance


        :returns: The fetched PortingBulkPortabilityInstance
        """
        return self._proxy.fetch()

    async def fetch_async(self) -> "PortingBulkPortabilityInstance":
        """
        Asynchronous coroutine to fetch the PortingBulkPortabilityInstance


        :returns: The fetched PortingBulkPortabilityInstance
        """
        return await self._proxy.fetch_async()

    def __repr__(self) -> str:
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        """
        context = " ".join("{}={}".format(k, v) for k, v in self._solution.items())
        return "<Twilio.Numbers.V1.PortingBulkPortabilityInstance {}>".format(context)


class PortingBulkPortabilityContext(InstanceContext):

    def __init__(self, version: Version, sid: str):
        """
        Initialize the PortingBulkPortabilityContext

        :param version: Version that contains the resource
        :param sid: A 34 character string that uniquely identifies the Portability check.
        """
        super().__init__(version)

        # Path Solution
        self._solution = {
            "sid": sid,
        }
        self._uri = "/Porting/Portability/{sid}".format(**self._solution)

    def fetch(self) -> PortingBulkPortabilityInstance:
        """
        Fetch the PortingBulkPortabilityInstance


        :returns: The fetched PortingBulkPortabilityInstance
        """

        payload = self._version.fetch(
            method="GET",
            uri=self._uri,
        )

        return PortingBulkPortabilityInstance(
            self._version,
            payload,
            sid=self._solution["sid"],
        )

    async def fetch_async(self) -> PortingBulkPortabilityInstance:
        """
        Asynchronous coroutine to fetch the PortingBulkPortabilityInstance


        :returns: The fetched PortingBulkPortabilityInstance
        """

        payload = await self._version.fetch_async(
            method="GET",
            uri=self._uri,
        )

        return PortingBulkPortabilityInstance(
            self._version,
            payload,
            sid=self._solution["sid"],
        )

    def __repr__(self) -> str:
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        """
        context = " ".join("{}={}".format(k, v) for k, v in self._solution.items())
        return "<Twilio.Numbers.V1.PortingBulkPortabilityContext {}>".format(context)


class PortingBulkPortabilityList(ListResource):

    def __init__(self, version: Version):
        """
        Initialize the PortingBulkPortabilityList

        :param version: Version that contains the resource

        """
        super().__init__(version)

        self._uri = "/Porting/Portability"

    def create(self, phone_numbers: List[str]) -> PortingBulkPortabilityInstance:
        """
        Create the PortingBulkPortabilityInstance

        :param phone_numbers: The phone numbers which portability is to be checked. This should be a list of strings. Phone numbers are in E.164 format (e.g. +16175551212). .

        :returns: The created PortingBulkPortabilityInstance
        """

        data = values.of(
            {
                "PhoneNumbers": serialize.map(phone_numbers, lambda e: e),
            }
        )

        payload = self._version.create(
            method="POST",
            uri=self._uri,
            data=data,
        )

        return PortingBulkPortabilityInstance(self._version, payload)

    async def create_async(
        self, phone_numbers: List[str]
    ) -> PortingBulkPortabilityInstance:
        """
        Asynchronously create the PortingBulkPortabilityInstance

        :param phone_numbers: The phone numbers which portability is to be checked. This should be a list of strings. Phone numbers are in E.164 format (e.g. +16175551212). .

        :returns: The created PortingBulkPortabilityInstance
        """

        data = values.of(
            {
                "PhoneNumbers": serialize.map(phone_numbers, lambda e: e),
            }
        )

        payload = await self._version.create_async(
            method="POST",
            uri=self._uri,
            data=data,
        )

        return PortingBulkPortabilityInstance(self._version, payload)

    def get(self, sid: str) -> PortingBulkPortabilityContext:
        """
        Constructs a PortingBulkPortabilityContext

        :param sid: A 34 character string that uniquely identifies the Portability check.
        """
        return PortingBulkPortabilityContext(self._version, sid=sid)

    def __call__(self, sid: str) -> PortingBulkPortabilityContext:
        """
        Constructs a PortingBulkPortabilityContext

        :param sid: A 34 character string that uniquely identifies the Portability check.
        """
        return PortingBulkPortabilityContext(self._version, sid=sid)

    def __repr__(self) -> str:
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        """
        return "<Twilio.Numbers.V1.PortingBulkPortabilityList>"



r"""
    This code was generated by
   ___ _ _ _ _ _    _ ____    ____ ____ _    ____ ____ _  _ ____ ____ ____ ___ __   __
    |  | | | | |    | |  | __ |  | |__| | __ | __ |___ |\ | |___ |__/ |__|  | |  | |__/
    |  |_|_| | |___ | |__|    |__| |  | |    |__] |___ | \| |___ |  \ |  |  | |__| |  \

    Twilio - Numbers
    This is the public Twilio REST API.

    NOTE: This class is auto generated by OpenAPI Generator.
    https://openapi-generator.tech
    Do not edit the class manually.
"""

from datetime import date
from typing import Any, Dict, List, Optional
from twilio.base import deserialize
from twilio.base.instance_context import InstanceContext
from twilio.base.instance_resource import InstanceResource
from twilio.base.list_resource import ListResource
from twilio.base.version import Version


class PortingPortInFetchInstance(InstanceResource):
    """
    :ivar port_in_request_sid: The SID of the Port In request. This is a unique identifier of the port in request.
    :ivar url: The URL of this Port In request
    :ivar account_sid: The Account SID that the numbers will be added to after they are ported into Twilio.
    :ivar notification_emails: List of emails for getting notifications about the LOA signing process. Allowed Max 10 emails.
    :ivar target_port_in_date: Minimum number of days in the future (at least 2 days) needs to be established with the Ops team for validation.
    :ivar target_port_in_time_range_start: Minimum hour in the future needs to be established with the Ops team for validation.
    :ivar target_port_in_time_range_end: Maximum hour in the future needs to be established with the Ops team for validation.
    :ivar port_in_request_status: The status of the port in request. The possible values are: In progress, Completed, Expired, In review, Waiting for Signature, Action Required, and Canceled.
    :ivar losing_carrier_information: The information for the losing carrier.
    :ivar phone_numbers: The list of phone numbers to Port in. Phone numbers are in E.164 format (e.g. +16175551212).
    :ivar documents: The list of documents SID referencing a utility bills
    """

    def __init__(
        self,
        version: Version,
        payload: Dict[str, Any],
        port_in_request_sid: Optional[str] = None,
    ):
        super().__init__(version)

        self.port_in_request_sid: Optional[str] = payload.get("port_in_request_sid")
        self.url: Optional[str] = payload.get("url")
        self.account_sid: Optional[str] = payload.get("account_sid")
        self.notification_emails: Optional[List[str]] = payload.get(
            "notification_emails"
        )
        self.target_port_in_date: Optional[date] = deserialize.iso8601_date(
            payload.get("target_port_in_date")
        )
        self.target_port_in_time_range_start: Optional[str] = payload.get(
            "target_port_in_time_range_start"
        )
        self.target_port_in_time_range_end: Optional[str] = payload.get(
            "target_port_in_time_range_end"
        )
        self.port_in_request_status: Optional[str] = payload.get(
            "port_in_request_status"
        )
        self.losing_carrier_information: Optional[Dict[str, object]] = payload.get(
            "losing_carrier_information"
        )
        self.phone_numbers: Optional[List[Dict[str, object]]] = payload.get(
            "phone_numbers"
        )
        self.documents: Optional[List[str]] = payload.get("documents")

        self._solution = {
            "port_in_request_sid": port_in_request_sid or self.port_in_request_sid,
        }
        self._context: Optional[PortingPortInFetchContext] = None

    @property
    def _proxy(self) -> "PortingPortInFetchContext":
        """
        Generate an instance context for the instance, the context is capable of
        performing various actions. All instance actions are proxied to the context

        :returns: PortingPortInFetchContext for this PortingPortInFetchInstance
        """
        if self._context is None:
            self._context = PortingPortInFetchContext(
                self._version,
                port_in_request_sid=self._solution["port_in_request_sid"],
            )
        return self._context

    def fetch(self) -> "PortingPortInFetchInstance":
        """
        Fetch the PortingPortInFetchInstance


        :returns: The fetched PortingPortInFetchInstance
        """
        return self._proxy.fetch()

    async def fetch_async(self) -> "PortingPortInFetchInstance":
        """
        Asynchronous coroutine to fetch the PortingPortInFetchInstance


        :returns: The fetched PortingPortInFetchInstance
        """
        return await self._proxy.fetch_async()

    def __repr__(self) -> str:
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        """
        context = " ".join("{}={}".format(k, v) for k, v in self._solution.items())
        return "<Twilio.Numbers.V1.PortingPortInFetchInstance {}>".format(context)


class PortingPortInFetchContext(InstanceContext):

    def __init__(self, version: Version, port_in_request_sid: str):
        """
        Initialize the PortingPortInFetchContext

        :param version: Version that contains the resource
        :param port_in_request_sid: The SID of the Port In request. This is a unique identifier of the port in request.
        """
        super().__init__(version)

        # Path Solution
        self._solution = {
            "port_in_request_sid": port_in_request_sid,
        }
        self._uri = "/Porting/PortIn/{port_in_request_sid}".format(**self._solution)

    def fetch(self) -> PortingPortInFetchInstance:
        """
        Fetch the PortingPortInFetchInstance


        :returns: The fetched PortingPortInFetchInstance
        """

        payload = self._version.fetch(
            method="GET",
            uri=self._uri,
        )

        return PortingPortInFetchInstance(
            self._version,
            payload,
            port_in_request_sid=self._solution["port_in_request_sid"],
        )

    async def fetch_async(self) -> PortingPortInFetchInstance:
        """
        Asynchronous coroutine to fetch the PortingPortInFetchInstance


        :returns: The fetched PortingPortInFetchInstance
        """

        payload = await self._version.fetch_async(
            method="GET",
            uri=self._uri,
        )

        return PortingPortInFetchInstance(
            self._version,
            payload,
            port_in_request_sid=self._solution["port_in_request_sid"],
        )

    def __repr__(self) -> str:
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        """
        context = " ".join("{}={}".format(k, v) for k, v in self._solution.items())
        return "<Twilio.Numbers.V1.PortingPortInFetchContext {}>".format(context)


class PortingPortInFetchList(ListResource):

    def __init__(self, version: Version):
        """
        Initialize the PortingPortInFetchList

        :param version: Version that contains the resource

        """
        super().__init__(version)

    def get(self, port_in_request_sid: str) -> PortingPortInFetchContext:
        """
        Constructs a PortingPortInFetchContext

        :param port_in_request_sid: The SID of the Port In request. This is a unique identifier of the port in request.
        """
        return PortingPortInFetchContext(
            self._version, port_in_request_sid=port_in_request_sid
        )

    def __call__(self, port_in_request_sid: str) -> PortingPortInFetchContext:
        """
        Constructs a PortingPortInFetchContext

        :param port_in_request_sid: The SID of the Port In request. This is a unique identifier of the port in request.
        """
        return PortingPortInFetchContext(
            self._version, port_in_request_sid=port_in_request_sid
        )

    def __repr__(self) -> str:
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        """
        return "<Twilio.Numbers.V1.PortingPortInFetchList>"
